/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package gdexpress.demofacturador.lib;

import DocumentoDTE.SiiDte.CdgItem;
import DocumentoDTE.SiiDte.DTEDefType;
import DocumentoDTE.SiiDte.DTEType;
import DocumentoDTE.SiiDte.DTE_Choice;
import DocumentoDTE.SiiDte.Detalle;
import DocumentoDTE.SiiDte.Detalle_IndExe;
import DocumentoDTE.SiiDte.Emisor;
import DocumentoDTE.SiiDte.Encabezado;
import DocumentoDTE.SiiDte.IdDoc;
import DocumentoDTE.SiiDte.Receptor;
import DocumentoDTE.SiiDte.Referencia;
import DocumentoDTE.SiiDte.Totales;
import com.liquid_technologies.ltxmllib12.DateTime;
import com.liquid_technologies.ltxmllib12.exceptions.LtException;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * Representa la estructura de un Documento.
 * @author yrios
 */
public class Documento implements IFactura {

    /**
     * Lista de Detalles de un Documento.
     */
    private List listaDetalles;
    
    /**
     * Lista de Referencias de un Documento.
     */
    private List listaReferencias;
    
    /**
     * DTE Documento.
     */
    private DocumentoDTE.SiiDte.DTEDefType dte;
    
    
    public DocumentoDTE.SiiDte.Documento getValor() {
        return dte.getDTE_Choice().getDocumento();
    }
    
    public DocumentoDTE.SiiDte.IdDoc getIdDoc(){
        return dte.getDTE_Choice().getDocumento().getEncabezado().getIdDoc();
    }
        
    public DocumentoDTE.SiiDte.Emisor getEmisor(){
        return dte.getDTE_Choice().getDocumento().getEncabezado().getEmisor();
    }            
         
    public DocumentoDTE.SiiDte.Receptor getReceptor(){
        return dte.getDTE_Choice().getDocumento().getEncabezado().getReceptor();
    }
    
    public DocumentoDTE.SiiDte.Totales getTotales(){
        return dte.getDTE_Choice().getDocumento().getEncabezado().getTotales();
    }
    
    /**
     * Crea un Objeto Documento.
     */
    public Documento() throws LtException {
        listaDetalles = new ArrayList();
        listaReferencias = new ArrayList();
        
        dte = new DTEDefType();
        dte.setDTE_Choice(new DTE_Choice());
        dte.getDTE_Choice().setDocumento(new DocumentoDTE.SiiDte.Documento());
        
        Encabezado encabezado = new Encabezado();
        encabezado.setIdDoc(new IdDoc());
        encabezado.setEmisor(new Emisor());
        encabezado.setReceptor(new Receptor());
        encabezado.setTotales(new Totales());
        dte.getDTE_Choice().getDocumento().setEncabezado(encabezado);
    }

    public List getListaDetalles() {
        return listaDetalles;
    }

    public void setListaDetalles(List listaDetalles) {
        this.listaDetalles = listaDetalles;
    }

    public List getListaReferencias() {
        return listaReferencias;
    }

    public void setListaReferencias(List listaReferencias) {
        this.listaReferencias = listaReferencias;
    }

    public DTEDefType getDte() {
        return dte;
    }
    
    /**
     * Establece los valores para Encabezado/IdDoc.
     * 
     * @param tipoDTE Tipo de DTE
     * @param folio Folio del documento
     * @param fchEmis Fecha de emisión
     */
    public void setEncabezadoIdDoc(String tipoDTE, BigInteger folio, DateTime fchEmis) throws LtException {
        IdDoc idDoc = getIdDoc();
        idDoc.setFolio(folio);
        DTEType tipoDte = DocumentoDTE.SiiDte.DTEType.fromString(tipoDTE);
        idDoc.setTipoDTE(tipoDte);
        idDoc.setFchEmis(fchEmis);
    }
    
    /**
     * Establece los valores para Encabezado/Emisor.
     * 
     * @param RUTEmisor RUT del emisor
     * @param rznSoc Razón social del emisor
     * @param giroEmis Giro del emisor
     * @param actectos Actectos del emisor
     * @param dirOrigen Dirección origen del emisor
     * @param cmnaOrigen Comuna origen del emisor
     * @param ciudadOrigen Ciudad origen del emisor
     */
    public void setEncabezadoEmisor(String RUTEmisor, String rznSoc, String giroEmis, String actecos, String dirOrigen, String cmnaOrigen, String ciudadOrigen) throws LtException {
        Emisor emisor = getEmisor();
        emisor.setRUTEmisor(RUTEmisor);
        emisor.setRznSoc(rznSoc);
        emisor.setGiroEmis(giroEmis);
        emisor.setDirOrigen(dirOrigen);
        emisor.setCmnaOrigen(cmnaOrigen);
        emisor.setCiudadOrigen(ciudadOrigen);
        
        emisor.getActeco().clear();
        String[] arregloActecos = actecos.split(",");
        for (int i = 0; i < arregloActecos.length; i++) {
            try {
                String act = arregloActecos[i];
                emisor.getActeco().add(new BigInteger(act));
            } catch (Exception e) {
                throw new LtException("El campo Acteco no es válido.");
            }
        }
    }
    
    /**
     * Establece los valores para Encabezado/Receptor.
     * 
     * @param RUTRecep RUT del Receptor
     * @param rznSocRecep Razón social del Receptor
     * @param giroRecep Giro del Receptor
     * @param dirRecep Dirección del Receptor
     * @param cmnaRecep Comuna del Receptor
     * @param ciudadRecep Ciudad del Receptor
     */
    public void setEncabezadoReceptor(
            String RUTRecep,
            String rznSocRecep,
            String giroRecep,
            String dirRecep,
            String cmnaRecep,
            String ciudadRecep) throws LtException{
        Receptor receptor = getReceptor();
        receptor.setRUTRecep(RUTRecep);
        receptor.setRznSocRecep(rznSocRecep);
        receptor.setGiroRecep(giroRecep);
        receptor.setDirRecep(dirRecep);
        receptor.setCmnaRecep(cmnaRecep);
        receptor.setCiudadRecep(ciudadRecep);
    }
    
    /**
     * Establece los valores para Encabezado/Totales.
     * 
     * @param mntTotal Monto total
     * @param mntExe Monto exento
     * @param mntNeto Monto neto
     * @param IVA IVA
     */
    public void setEncabezadoTotales(BigInteger mntTotal, BigInteger mntExe, BigInteger mntNeto, BigInteger IVA) throws LtException {
        Totales totales = getTotales();
        totales.setMntTotal(mntTotal);
        totales.setMntExe(mntExe);
        totales.setMntNeto(mntNeto);
        totales.setIVA(IVA);
    }
    
    /**
     * Adiciona un item a los detalles del documento.
     * 
     * @param nmbItem Nombre del item
     * @param montoItem Monto del item
     * @param indExe Indicador exento
     * @param dscItem Descripción del item
     * @param qtyItem Cantidad del item
     * @param unmdItem Unidad de medida del item
     * @param prcItem Precio del item
     * @param listaCdgItem Lista de códigos del item
     */
    public void adicionarDetalle(
            String nmbItem,
            BigInteger montoItem,
            String indExe,
            String dscItem,
            BigDecimal qtyItem,
            String unmdItem,
            BigDecimal prcItem,
            List listaCdgItem) throws LtException {
        actualizarDetalleEnIndice(
            nmbItem,
            montoItem,
            indExe,
            dscItem,
            qtyItem,
            unmdItem,
            prcItem,
            listaCdgItem, -1);
    }
    
    /**
     * Adiciona o actualiza un item de los detalles del documento dada la posición del mismo.
     * 
     * @param nmbItem Nombre del item
     * @param montoItem Monto del item
     * @param indExe Indicador exento
     * @param dscItem Descripción del item
     * @param qtyItem Cantidad del item
     * @param unmdItem Unidad de medida del item
     * @param prcItem Precio del item
     * @param listaCdgItem Lista de códigos del item
     * @param indice indice del item a actualizar
     */
    public void actualizarDetalleEnIndice(
            String nmbItem,
            BigInteger montoItem,
            String indExe,
            String dscItem,
            BigDecimal qtyItem,
            String unmdItem,
            BigDecimal prcItem,
            List listaCdgItem,
            int indice) throws LtException {
        Detalle detalle = new Detalle();
        detalle.setNroLinDet(new BigInteger(String.valueOf(listaDetalles.size() + 1)));
        detalle.setNmbItem(nmbItem);
        detalle.setMontoItem(montoItem);
        if (!indExe.isEmpty()) {
            Detalle_IndExe detalle_IndExe = DocumentoDTE.SiiDte.Detalle_IndExe.fromString(indExe);
            detalle.setIndExe(detalle_IndExe);
        }
        detalle.setDscItem(dscItem);
        detalle.setQtyItem(qtyItem);
        detalle.setUnmdItem(unmdItem);
        detalle.setPrcItem(prcItem);
        
        detalle.getCdgItem().clear();
        for (Iterator it = listaCdgItem.iterator(); it.hasNext();) {
            CdgItem cdgItem = (CdgItem)it.next();
            detalle.getCdgItem().add(cdgItem);
        }
        
        if (indice > -1) {
            listaDetalles.set(indice, detalle);
        } else {
            listaDetalles.add(detalle);
        }
    }
    
    /**
     * Elimina un item de la lista de detalles.
     * 
     * @param indice indice del item a eliminar
     */
    public void eliminarDetalleEnIndice(int indice) {
        listaDetalles.remove(indice);
    }
    
    /**
     * Crea un objeto CdgItem.
     * 
     * @param tpoCodigo Tipo de código
     * @param vlrCodigo Valor del código
     * @return CdgItem objeto CdgItem
     */
    public CdgItem crearDetalleCdgItem(
            String tpoCodigo,
            String vlrCodigo) throws LtException {
        CdgItem cdgItem = new CdgItem();
        cdgItem.setTpoCodigo(tpoCodigo);
        cdgItem.setVlrCodigo(vlrCodigo);
        
        return cdgItem;
    }
    
    /**
     * Adiciona una Referencia a la lista de Referencias.
     * 
     * @param tpoDocRef Tipo de documento de la Referencia
     * @param folioRef Folio de la Referencia
     * @param fchRef Fecha de la Referencia
     * @param razonRef Razón de la Referencia
     */
    public void adicionarReferencia(
            String tpoDocRef,
            String folioRef,
            DateTime fchRef,
            String razonRef) throws LtException {
        actualizarReferenciaEnIndice(tpoDocRef, folioRef, fchRef, razonRef, -1);
    }
    
    /**
     * Actualiza o adiciona una Referencia a la lista de Referencias dado el indice de la misma.
     * 
     * @param tpoDocRef Tipo de documento de la Referencia
     * @param folioRef Folio de la Referencia
     * @param fchRef Fecha de la Referencia
     * @param razonRef Razón de la Referencia
     * @param indice indice de la Referencia a actualizar
     */
    public void actualizarReferenciaEnIndice(
            String tpoDocRef,
            String folioRef,
            DateTime fchRef,
            String razonRef,
            int indice) throws LtException {
        Referencia referencia = new Referencia();
        referencia.setNroLinRef(new BigInteger(String.valueOf(listaReferencias.size() +1)));
        referencia.setTpoDocRef(tpoDocRef);
        referencia.setFolioRef(folioRef);
        referencia.setFchRef(fchRef);
        referencia.setRazonRef(razonRef);
        if (indice > -1) {
            listaReferencias.set(indice, referencia);
        } else {
            listaReferencias.add(referencia);
        }
    }
    
    /**
     * Elimina una Referencia de la lista de Referencias dado el indice de la misma.
     * 
     * @param indice indice de la Referencia en la lista
     */
    public void eliminarReferenciaEnIndice(int indice) {
        listaReferencias.remove(indice);
    }
    
    /*
     * Obtiene el XML representativo del Documento.
     * 
     * @return String con el XML del Documento.
     */
    public String obtenerXML() throws Exception {
        int indice = 0;

        //Documento/Detalle
        getValor().getDetalle().clear();
        for (Iterator it = listaDetalles.iterator(); it.hasNext();) {
            Detalle detalle = (Detalle)it.next();
            detalle.setNroLinDet(new BigInteger(String.valueOf(++indice)));
            getValor().getDetalle().add(detalle);
        }
        
        //Documento/Referencia
        getValor().getReferencia().clear();
        indice = 0;
        for (Iterator it = listaReferencias.iterator(); it.hasNext();) {
            Referencia referencia = (Referencia)it.next();
            referencia.setNroLinRef(new BigInteger(String.valueOf(++indice)));
            getValor().getReferencia().add(referencia);
        }
        
        return GeneradorXmlDTE.obtenerXML(dte);
    }
    
}
