/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package gdexpress.demofacturador.lib;

import DocumentoDTE.EnvioDTE_v10Lib.FRMTA_Algoritmo;
import DocumentoDTE.EnvioDTE_v10Lib.FRMTB_Algoritmo;
import DocumentoDTE.EnvioDTE_v10Lib.FRMT_Algoritmo;
import DocumentoDTE.SiiDte.DTEDefType;
import DocumentoDTE.SiiDte.Documento;
import DocumentoDTE.SiiDte.Exportaciones;
import DocumentoDTE.SiiDte.Liquidacion;
import com.liquid_technologies.ltxmllib12.BinaryData;
import com.liquid_technologies.ltxmllib12.DateTime;
import com.liquid_technologies.ltxmllib12.EOLType;
import com.liquid_technologies.ltxmllib12.Encoding;
import com.liquid_technologies.ltxmllib12.Formatting;
import com.liquid_technologies.ltxmllib12.XmlSerializationContext;
import com.liquid_technologies.ltxmllib12.exceptions.LtException;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.math.BigInteger;

/**
 * Clase utilitaria para generar el XML de los documentos DTE.
 * 
 * @author yrios
 */
public class GeneradorXmlDTE {
    
    /**
     * Obtiene el XML de un DTE.
     * @param dte DTE al cual se le va a obtener el XML
     * @return String que representa el XML del DTE
     * @throws LtException 
     */
    public static String obtenerXML(DTEDefType dte) throws LtException, UnsupportedEncodingException, IOException {
        dte.setVersion(new BigDecimal("1.0"));
        
        if (dte.getDTE_Choice().getDocumento() != null) {
            llenarDocumento(dte.getDTE_Choice().getDocumento());
        } else if (dte.getDTE_Choice().getExportaciones() != null) {
            llenarExportaciones(dte.getDTE_Choice().getExportaciones());
        } else if (dte.getDTE_Choice().getLiquidacion() != null) {
            llenarLiquidacion(dte.getDTE_Choice().getLiquidacion());
        }
        
        XmlSerializationContext xsc = new XmlSerializationContext();
        xsc.getNamespaceAliases().add("ABCXYZ", dte.getNamespace());
        
        String xml = dte.toXml(true, Formatting.Indented, Encoding.UTF8, EOLType.CRLF, xsc);
        String xmlFinal = xml.replace(":ABCXYZ", "").replace("ABCXYZ:", "")
                .replace("<!--Created by Liquid XML Data Binding Libraries (www.liquid-technologies.com) for GDE S.A. (1 * Developer Bundle)-->\r\n", "")
                .replace("<!--Created by Liquid XML Data Binding Libraries (www.liquid-technologies.com) for SOUTH CONSULTING SIGNATURE S.A. (1 * Developer Edition)-->\r\n", "")
                .replace("xmlns=\"http://www.sii.cl/SiiDte\" ", "")
                .replace("DTEDefType", "DTE");
        return xmlFinal;
    }
    
    /**
     * Llena la estructura TED de un DTE de tipo Documento.
     * 
     * @param doc Documento al cual se le va a llenar la estructura TED
     */
    private static void llenarDocumento(Documento doc) throws LtException {
        doc.setID("ID_1");
        
        //TED
        doc.setTED(new DocumentoDTE.SiiDte.TED());
        doc.getTED().setVersion("1.0");
        
        //TED/DD
        doc.getTED().setDD(new DocumentoDTE.SiiDte.DD());
        doc.getTED().getDD().setRE(doc.getEncabezado().getEmisor().getRUTEmisor());
        doc.getTED().getDD().setTD(doc.getEncabezado().getIdDoc().getTipoDTE());
        doc.getTED().getDD().setF(doc.getEncabezado().getIdDoc().getFolio());
        
        doc.getTED().getDD().setFE(doc.getEncabezado().getIdDoc().getFchEmis());
        doc.getTED().getDD().setRR(doc.getEncabezado().getReceptor().getRUTRecep());
        doc.getTED().getDD().setRSR("RSR");
        doc.getTED().getDD().setMNT(doc.getEncabezado().getTotales().getMntTotal().longValue());
        doc.getTED().getDD().setIT1("IT1");
        
        //TED/Fake CAF
        doc.getTED().getDD().setCAF(new DocumentoDTE.SiiDte.CAF());
        doc.getTED().getDD().getCAF().setDA(new DocumentoDTE.SiiDte.DA());
        
        doc.getTED().getDD().getCAF().getDA().setRE("00000000-K");
        doc.getTED().getDD().getCAF().getDA().setRS("A");
        doc.getTED().getDD().getCAF().getDA().setTD(DocumentoDTE.SiiDte.DTEType.n33);
        doc.getTED().getDD().getCAF().getDA().setFA(new DateTime((short)2010, (byte)1, (byte)1));
        doc.getTED().getDD().getCAF().getDA().setIDK(0);
        
        doc.getTED().getDD().getCAF().getDA().setDA_Choice(new DocumentoDTE.SiiDte.DA_Choice());
        doc.getTED().getDD().getCAF().getDA().getDA_Choice().setRSAPK(new DocumentoDTE.SiiDte.RSAPK());
        doc.getTED().getDD().getCAF().getDA().getDA_Choice().getRSAPK().setM(new BinaryData(new byte[] { }));
        doc.getTED().getDD().getCAF().getDA().getDA_Choice().getRSAPK().setE(new BinaryData(new byte[] { }));
        
        doc.getTED().getDD().getCAF().getDA().setRNG(new DocumentoDTE.SiiDte.RNG());
        doc.getTED().getDD().getCAF().getDA().getRNG().setD(BigInteger.ONE);
        doc.getTED().getDD().getCAF().getDA().getRNG().setH(BigInteger.ONE);
        
        doc.getTED().getDD().getCAF().setFRMA(new DocumentoDTE.SiiDte.FRMA());
        doc.getTED().getDD().getCAF().getFRMA().setAlgoritmo("SHA1withRSA");
        doc.getTED().getDD().getCAF().getFRMA().setPrimitiveValue(new BinaryData(new byte[] { }));

        doc.getTED().getDD().setTSTED(new DateTime((short)2010, (byte)1, (byte)1, (byte)0, (byte)0, (byte)0));

        doc.getTED().setFRMT(new DocumentoDTE.SiiDte.FRMT());
        doc.getTED().getFRMT().setAlgoritmo(FRMT_Algoritmo.SHA1withRSA);
        doc.getTED().getFRMT().setPrimitiveValue(new BinaryData(new byte[] { }));

        doc.setTmstFirma(new DateTime((short)2010, (byte)1, (byte)1, (byte)0, (byte)0, (byte)0));  
    }
    
    /**
     * Llena la estructura TED de un DTE de tipo Exportaciones.
     * 
     * @param doc Exportación a la cual se le va a llenar la estructura TED
     */
    private static void llenarExportaciones(Exportaciones doc) throws LtException {
        doc.setID("ID_1");
        
        //TED
        doc.setTED(new DocumentoDTE.SiiDte.TEDB());
        doc.getTED().setVersion("1.0");
        
        //TED/DD
        doc.getTED().setDD(new DocumentoDTE.SiiDte.DDB());
        doc.getTED().getDD().setRE(doc.getEncabezado().getEmisor().getRUTEmisor());
        doc.getTED().getDD().setTD(doc.getEncabezado().getIdDoc().getTipoDTE());
        doc.getTED().getDD().setF(doc.getEncabezado().getIdDoc().getFolio());
        
        doc.getTED().getDD().setFE(doc.getEncabezado().getIdDoc().getFchEmis());
        doc.getTED().getDD().setRR(doc.getEncabezado().getReceptor().getRUTRecep());
        doc.getTED().getDD().setRSR("RSR");
        doc.getTED().getDD().setMNT(doc.getEncabezado().getTotales().getMntTotal());
        doc.getTED().getDD().setIT1("IT1");
        
        //TED/Fake CAF
        doc.getTED().getDD().setCAF(new DocumentoDTE.SiiDte.CAFB());
        doc.getTED().getDD().getCAF().setDA(new DocumentoDTE.SiiDte.DAB());
        
        doc.getTED().getDD().getCAF().getDA().setRE("00000000-K");
        doc.getTED().getDD().getCAF().getDA().setRS("A");
        doc.getTED().getDD().getCAF().getDA().setTD(DocumentoDTE.SiiDte.EXPType.n110);
        doc.getTED().getDD().getCAF().getDA().setFA(new DateTime((short)2010, (byte)1, (byte)1));
        doc.getTED().getDD().getCAF().getDA().setIDK(0);
        
        doc.getTED().getDD().getCAF().getDA().setDA1_Choice(new DocumentoDTE.SiiDte.DA1_Choice());
        doc.getTED().getDD().getCAF().getDA().getDA1_Choice().setRSAPK(new DocumentoDTE.SiiDte.RSAPKB());
        doc.getTED().getDD().getCAF().getDA().getDA1_Choice().getRSAPK().setM(new BinaryData(new byte[] { }));
        doc.getTED().getDD().getCAF().getDA().getDA1_Choice().getRSAPK().setE(new BinaryData(new byte[] { }));
        
        doc.getTED().getDD().getCAF().getDA().setRNG(new DocumentoDTE.SiiDte.RNGB());
        doc.getTED().getDD().getCAF().getDA().getRNG().setD(BigInteger.ONE);
        doc.getTED().getDD().getCAF().getDA().getRNG().setH(BigInteger.ONE);
        
        doc.getTED().getDD().getCAF().setFRMA(new DocumentoDTE.SiiDte.FRMAB());
        doc.getTED().getDD().getCAF().getFRMA().setAlgoritmo("SHA1withRSA");
        doc.getTED().getDD().getCAF().getFRMA().setPrimitiveValue(new BinaryData(new byte[] { }));

        doc.getTED().getDD().setTSTED(new DateTime((short)2010, (byte)1, (byte)1, (byte)0, (byte)0, (byte)0));

        doc.getTED().setFRMT(new DocumentoDTE.SiiDte.FRMTB());
        doc.getTED().getFRMT().setAlgoritmo(FRMTB_Algoritmo.SHA1withRSA);
        doc.getTED().getFRMT().setPrimitiveValue(new BinaryData(new byte[] { }));

        doc.setTmstFirma(new DateTime((short)2010, (byte)1, (byte)1, (byte)0, (byte)0, (byte)0));  
    }
    
    /**
     * Llena la estructura TED de un DTE de tipo Liquidación.
     * 
     * @param doc Liquidación a la cual se le va a llenar la estructura TED
     */
    private static void llenarLiquidacion(Liquidacion doc) throws LtException {
        doc.setID("ID_1");
        
        //TED
        doc.setTED(new DocumentoDTE.SiiDte.TEDA());
        doc.getTED().setVersion("1.0");
        
        //TED/DD
        doc.getTED().setDD(new DocumentoDTE.SiiDte.DDA());
        doc.getTED().getDD().setRE(doc.getEncabezado().getEmisor().getRUTEmisor());
        doc.getTED().getDD().setTD(doc.getEncabezado().getIdDoc().getTipoDTE());
        doc.getTED().getDD().setF(doc.getEncabezado().getIdDoc().getFolio());
        
        doc.getTED().getDD().setFE(doc.getEncabezado().getIdDoc().getFchEmis());
        doc.getTED().getDD().setRR(doc.getEncabezado().getReceptor().getRUTRecep());
        doc.getTED().getDD().setRSR("RSR");
        doc.getTED().getDD().setMNT(doc.getEncabezado().getTotales().getMntTotal().longValue());
        doc.getTED().getDD().setIT1("IT1");
        
        //TED/Fake CAF
        doc.getTED().getDD().setCAF(new DocumentoDTE.SiiDte.CAFA());
        doc.getTED().getDD().getCAF().setDA(new DocumentoDTE.SiiDte.DAA());
        
        doc.getTED().getDD().getCAF().getDA().setRE("00000000-K");
        doc.getTED().getDD().getCAF().getDA().setRS("A");
        doc.getTED().getDD().getCAF().getDA().setTD(DocumentoDTE.SiiDte.LIQType.n43);
        doc.getTED().getDD().getCAF().getDA().setFA(new DateTime((short)2010, (byte)1, (byte)1));
        doc.getTED().getDD().getCAF().getDA().setIDK(0);
        
        doc.getTED().getDD().getCAF().getDA().setDA0_Choice(new DocumentoDTE.SiiDte.DA0_Choice());
        doc.getTED().getDD().getCAF().getDA().getDA0_Choice().setRSAPK(new DocumentoDTE.SiiDte.RSAPKA());
        doc.getTED().getDD().getCAF().getDA().getDA0_Choice().getRSAPK().setM(new BinaryData(new byte[] { }));
        doc.getTED().getDD().getCAF().getDA().getDA0_Choice().getRSAPK().setE(new BinaryData(new byte[] { }));
        
        doc.getTED().getDD().getCAF().getDA().setRNG(new DocumentoDTE.SiiDte.RNGA());
        doc.getTED().getDD().getCAF().getDA().getRNG().setD(BigInteger.ONE);
        doc.getTED().getDD().getCAF().getDA().getRNG().setH(BigInteger.ONE);
        
        doc.getTED().getDD().getCAF().setFRMA(new DocumentoDTE.SiiDte.FRMAA());
        doc.getTED().getDD().getCAF().getFRMA().setAlgoritmo("SHA1withRSA");
        doc.getTED().getDD().getCAF().getFRMA().setPrimitiveValue(new BinaryData(new byte[] { }));
        

        doc.getTED().getDD().setTSTED(new DateTime((short)2010, (byte)1, (byte)1, (byte)0, (byte)0, (byte)0));

        doc.getTED().setFRMT(new DocumentoDTE.SiiDte.FRMTA());
        doc.getTED().getFRMT().setAlgoritmo(FRMTA_Algoritmo.SHA1withRSA);
        doc.getTED().getFRMT().setPrimitiveValue(new BinaryData(new byte[] { }));

        doc.setTmstFirma(new DateTime((short)2010, (byte)1, (byte)1, (byte)0, (byte)0, (byte)0));  
    }
    
}
