/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package gdexpress.demofacturador.vista;

import com.liquid_technologies.ltxmllib12.DateTime;
import com.liquid_technologies.ltxmllib12.exceptions.LtInvalidStateException;
import gdexpress.demofacturador.lib.AsistenteEnum;
import gdexpress.demofacturador.lib.InfoMostrar;
import gdexpress.demofacturador.lib.ValidadorActeco;
import gdexpress.demofacturador.lib.ValidadorRUT;
import java.awt.Component;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;

/**
 * Clase utilitaria para el trabajo con la interfaz de usuario.
 * 
 * @author yrios
 */
public class UtilIU {
    
    /**
     * Llena un combo con los valores provenientes de un Enumerado.
     * 
     * @param combo combo a llenar
     * @param tipoEnum Tipo del Enumerado
     */
    public static void llenarComboDesdeEnum(JComboBox combo, Class tipoEnum) {
        List valoresMostrar = AsistenteEnum.obtenerValoresMostrarPara(tipoEnum);
        InfoMostrar[] arrInfoMostrar = new InfoMostrar[valoresMostrar.size()];
        combo.setModel(new DefaultComboBoxModel(valoresMostrar.toArray(arrInfoMostrar)));
    }
    
    /**
     * Llena un combo con los valores provenientes de una Lista.
     * 
     * @param combo combo a llenar
     * @param listaValores Lista de objetos InfoMostrar a cargar en el combo
     */
    public static void llenarComboDesdeLista(JComboBox combo, List listaValores) {
        if (listaValores != null) {
            InfoMostrar[] arrInfoMostrar = new InfoMostrar[listaValores.size()];
            combo.setModel(new DefaultComboBoxModel(listaValores.toArray(arrInfoMostrar)));
        } else {
            combo.setModel(new DefaultComboBoxModel());
        }
    }
    
    /**
     * Obtiene los componentes que se encuentran dentro de un componente contenedor.
     * 
     * @param componente componente contenedor
     * @param tipos Lista de clases con los tipos de los componentes a obtener
     * @return List<Component> Lista de objetos Component
     */
    public static List obtenerComponentes(JInternalFrame frame, Class[] tipos) {
        List listaComponentes = new ArrayList();
        Component[] componentes = frame.getContentPane().getComponents();
        
        for (int i = 0; i < componentes.length; i++) {
            Component comp = componentes[i];
            if (JScrollPane.class.isInstance(comp)) {
                listaComponentes.addAll(obtenerComponentesJScrollPane((JScrollPane)comp, tipos));
            } else {
                for (int j = 0; j < tipos.length; j++) {
                    Class tipo = tipos[j];
                    if (tipo.isInstance(comp)) {
                        listaComponentes.add(comp);
                        break;
                    }
                }
            }
        }
        
        return listaComponentes;
    }
    
    /**
     * Obtiene los componentes que estan dentro de un JScrollPane que coincidan con los tipos de componentes
     * pasados por parámetro.
     * 
     * @param jsp JScrollPane
     * @param tipos Lista de clases con los tipos de los componentes a obtener
     * @return List<Component> Lista de objetos Component
     */
    public static List obtenerComponentesJScrollPane(JScrollPane jsp, Class[] tipos) {
        List listaComponentes = new ArrayList();
        Component[] componentes = jsp.getViewport().getComponents();
        for (int i = 0; i < componentes.length; i++) {
            Component comp = componentes[i];
            
            for (int j = 0; j < tipos.length; j++) {
                Class tipo = tipos[j];
                if (tipo.isInstance(comp)) {
                    listaComponentes.add(comp);
                    break;
                }
            }
        }
        
        return listaComponentes;
    }
    
    /**
     * Permite realizar las validaciones sobre las diferentes secciones de un formulario.
     * 
     * @param form Formulario a validar
     * @param secciones Secciones del formulario a validar
     * @param definicionesValidacion HashMap<String, Validacion> con las definiciones de validaciones para el formulario
     * @return String con los mensajes de error producto de la validación
     */
    public static String validarFormulario(JInternalFrame form, List secciones, HashMap definicionesValidacion) {
        StringBuilder mensajeFinal = new StringBuilder();
        List componentes = obtenerComponentes(form, new Class[] {JComboBox.class, JTextField.class, JTable.class, JFormattedTextField.class});
        for (Iterator it = componentes.iterator(); it.hasNext();) {
            Component component = (Component)it.next();
            String valor = "";
            if (JComboBox.class.isInstance(component)) {
                valor = ((InfoMostrar)((JComboBox)component).getModel().getSelectedItem()).getValor();
            } else if (JTextField.class.isInstance(component)) {
                valor = ((JTextField)component).getText();
            } else if (JFormattedTextField.class.isInstance(component)) {
                valor = ((JFormattedTextField)component).getText();
            } else if (JTable.class.isInstance(component)) {
                int cantFilas = ((JTable)component).getRowCount();
                valor = cantFilas == 0 ? "" : String.valueOf(cantFilas);
            }
            
            String nombreControl = component.getName();
            if (definicionesValidacion.containsKey(nombreControl) && secciones.contains(((Validacion)definicionesValidacion.get(nombreControl)).getSeccion())) {
                Validacion defVal = (Validacion)definicionesValidacion.get(nombreControl);
                Set nombreReglas = defVal.getReglas().keySet();
                for (Iterator it1 = nombreReglas.iterator(); it1.hasNext();) {
                    String nombreRegla = (String)it1.next();
                    String mensaje = "";
                    if (nombreRegla.equals("req")) {
                        if (valor.isEmpty()) {
                            mensaje = "El campo {nombre} es requrido.\n".replace("{nombre}", defVal.getLabelCampo());
                            mensajeFinal.append(mensaje);
                            break;
                        }
                    } else if (nombreRegla.equals("num")) {
                        try {
                            if (!valor.isEmpty()) {
                                BigDecimal valorDec = new BigDecimal(valor);
                            }
                        } catch (Exception exc) {
                            mensaje = "El campo {nombre} solo admite valores numéricos.\n".replace("{nombre}", defVal.getLabelCampo());
                            mensajeFinal.append(mensaje);
                            break;
                        }
                    } else if (nombreRegla.equals("min")) {
                            if (!valor.isEmpty())
                            {
                                BigDecimal valorDec = new BigDecimal(valor);
                                BigDecimal decComp = new BigDecimal(defVal.getReglas().get(nombreRegla).toString());
                                
                                if (valorDec.compareTo(decComp) < 0)
                                {
                                    mensaje = "El valor mínimo para el campo {nombre} es {valor}.\n".replace("{nombre}", defVal.getLabelCampo()).replace("{valor}", decComp.toPlainString());
                                    mensajeFinal.append(mensaje);
                                    break;
                                }
                            }
                   } else if (nombreRegla.equals("rut")) {
                        if (!valor.isEmpty()) {
                            if (!ValidadorRUT.esValido(valor)) {
                                mensaje = "El valor del campo {nombre} no es un RUT válido.\n".replace("{nombre}", defVal.getLabelCampo());
                                mensajeFinal.append(mensaje);
                                break;
                            }
                        }
                    } else if (nombreRegla.equals("acteco")) {
                        if (!valor.isEmpty()) {
                            if (!ValidadorActeco.esValido(valor)) {
                                mensaje = "El valor del campo {nombre} no es válido.\n".replace("{nombre}", defVal.getLabelCampo());
                                mensajeFinal.append(mensaje);
                                break;
                            }
                        }
                    }
                }
            }
        }
        
        return mensajeFinal.toString();
    }
    
    /**
     * Convierte una fecha del tipo DateTime a String.
     * 
     * @param fecha fecha del tipo DateTime
     * @return String fecha convertida a String
     * @throws LtInvalidStateException 
     */
    public static String convertirFechaString(DateTime fecha) throws LtInvalidStateException {
        String fechaString = String.valueOf(fecha.getDay()) + "/" + String.valueOf(fecha.getMonth()) + "/" + String.valueOf(fecha.getYear());
        return fechaString;
    }
    
    /**
     * Pregunta al usuario para confirmar la eliminación de una fila en un grid.
     * 
     * @return true si confirma, sino false.
     */
    public static boolean confirmarEliminacionFilaGrid(Component frame) {
        String opcionSi = "Si";
        String opcionNo = "No";
        Object[] options = {opcionSi, opcionNo};
        int n = JOptionPane.showOptionDialog(frame,
                "Está seguro que desea eliminar?",
                "Eliminar",
                JOptionPane.YES_NO_OPTION,
                JOptionPane.QUESTION_MESSAGE,
                null, //No usa un icono oersoonalizado
                options, //Texto de los botones
                opcionSi); //Texto del boton por defecto
        if (n == JOptionPane.YES_OPTION) {
            return true;
        } else {
            return false;
        }
    }
    
}
